-----------------------------------------------------------------------------
-- Supported test and miscelaneous methods.
--
-- Copyright (c) 2008, 2009 Wim Langers. All rights reserved.
-- Licensed under the same terms as Lua itself.
-- Based on LuaUnit.lua v1.3 under X11 License
-- see LICENSE.txt
--
-- @release 1.5.1 - 16 Feb 2009
-----------------------------------------------------------------------------

-----------------------------------------------------------------------------
-- Class definition.
-----------------------------------------------------------------------------
local M = {class = 'btd.lua.TestApi'}
local MT = {__index = M,__metatable = {}}

-- Some people like assertEquals(actual,expected) and some people prefer
-- assertEquals(expected,actual).
M.USE_EXPECTED_ACTUAL_IN_ASSERT_EQUALS = true

-----------------------------------------------------------------------------
-- Assert that calling f with the arguments will raise an error
-- @param   f       function under test
-- @param   ...     parameters to function onder test
-- @usage           assertError(f,1,2) => f(1,2) should generate an error
-----------------------------------------------------------------------------
local function fails(f,...)
    local has_error,error_msg = not pcall(f,...)
    if has_error then return end 
    error("No error generated",2)
end

-----------------------------------------------------------------------------
-- Assert that two values are equal and calls error else
-- @param   actual      result of test
-- @param   expected    expected result of test
-- @usage               assertEquals(1 + 1,2) => 1 + 1 = 2 ?
-----------------------------------------------------------------------------
local function equals(actual,expected)
    if not M.USE_EXPECTED_ACTUAL_IN_ASSERT_EQUALS then expected,actual = actual,expected end
    local nesting = '\n'
    local function assertEqualsRec(actual,expected,key)
        if actual == expected then return '' end
        local errorMsg = ''
        if type(actual) == 'table' and type(expected) == 'table' then
            for k,v in pairs(expected) do
                if actual[k] == nil then
                    errorMsg = errorMsg..nesting..'actual   : key : '..k..' missing'
                else
                    errorMsg = errorMsg..assertEqualsRec(actual[k],v,k)
                end
            end
            for k,v in pairs(actual) do
                if expected[k] == nil then
                    errorMsg = errorMsg..nesting..'actual   : key : '..k..' not expected'
                end
            end
        else
            if key then
                errorMsg = errorMsg..nesting..'expected : '..key..' = '..((type(expected) == 'string') and "'"..expected.."'" or tostring(expected))
                errorMsg = errorMsg..nesting..'actual   : '..key..' = '..((type(actual) == 'string') and "'"..actual.."'" or tostring(actual))
            else
                errorMsg = errorMsg..nesting..'expected : '..((type(expected) == 'string') and "'"..expected.."'" or tostring(expected))
                if type(expected) == 'string' then errorMsg = errorMsg..nesting else errorMsg = errorMsg..', ' end
                errorMsg = errorMsg..'actual   : '..((type(actual) == 'string') and "'"..actual.."'" or tostring(actual))
            end
        end
        return errorMsg
    end
    local errorMsg = assertEqualsRec(actual,expected)
    if #errorMsg == 0 then return end
    error(errorMsg,2)
end

-----------------------------------------------------------------------------
-- Wrap a set of functions into a Runnable test class:
-- @param   ...     names of functions to be processed
-- @return          list of test methods
-- @usage           TestFunctions = wrapFunctions(f1,f2,f3,f3,f5)
--                  Now,TestFunctions will be picked up by LuaUnit:run()
-----------------------------------------------------------------------------
local function wrap(...)
    local testClass = {}
    for _,testMethod in ipairs{...} do testClass[testMethod] = _G[testMethod] end
    return testClass
end

-----------------------------------------------------------------------------
-- Exported functions.
-----------------------------------------------------------------------------
M.equals = equals
assertEquals = equals -- to be deprecated ?
assert_Equals = equals -- to be deprecated ?
M.fails = fails
assertError = fails -- to be deprecated ?
assert_error = fails -- to be deprecated ?
M.wrap = wrap
wrapFunctions = wrap -- to be deprecated ?
return M

